package jp.co.kayo.android.localplayer;

import java.util.Timer;
import java.util.TimerTask;

import jp.co.kayo.android.localplayer.appwidget.AppWidgetHelper;
import jp.co.kayo.android.localplayer.service.MediaPlayerService;

import android.app.Application;
import android.content.Intent;

public class MyApplication extends Application {
    private long mSleepTimerStart;
    private long mSleepTimerEnd;
    private Timer mTimer = null;
    
    public boolean isSetTimer(){
        return mSleepTimerEnd>0;
    }
    
    public boolean isTimerOn(){
        return mTimer != null;
    }
    
    public void startSleepTimer(long delay){
        cancelTimer();
        mSleepTimerEnd = delay;
        startTimer(delay);
    }
    
    public void cancelSleepTimer(){
        cancelTimer();
        mSleepTimerEnd = 0;
        mSleepTimerStart = 0;
    }
    
    public void pauseSleepTimer(){
        cancelTimer();
        mSleepTimerEnd = getCalcLimit();
    }
    
    public void resumeSleepTimer(){
        cancelTimer();
        startTimer(mSleepTimerEnd);
    }
    
    public long getLimit(){
        if(isTimerOn()){
            return getCalcLimit();
        }else{
            return mSleepTimerEnd;
        }
    }
    
    public long getCalcLimit(){
        if(mSleepTimerEnd>0){
            long now = System.currentTimeMillis();
            long limit = (mSleepTimerStart+mSleepTimerEnd) - now;
            if(limit>0){
                return limit;
            }
        }
        return 0;
    }
    
    
    private void cancelTimer(){
        if(mTimer!=null){
            mTimer.cancel();
            mTimer = null;
        }
    }
    
    private void startTimer(long delay){
        mSleepTimerStart = System.currentTimeMillis();
        mTimer = new Timer(true);
        mTimer.schedule(new TimerTask(){
            @Override
            public void run() {
                Intent i = new Intent(MyApplication.this, MediaPlayerService.class);
                i.setAction(AppWidgetHelper.CALL_STOP);
                startService(i);
                cancelSleepTimer();
            }}, delay);
    }
    
}
