package jp.co.kayo.android.localplayer.plugin.sleeptimer;

import java.util.Timer;
import java.util.TimerTask;

import jp.co.kayo.android.localplayer.MyApplication;
import jp.co.kayo.android.localplayer.R;
import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.TimePicker;

public class MainActivity extends Activity implements OnClickListener {
    private MyApplication myApp;
    private TextView textTimer;
    private Button btnStart;
    private Button btnSleepCancel;
    private Button btnSleepPauseResume;
    private TimePicker timePicker1;
    private Timer mTimer;
    private Handler mHandler = new Handler();
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main_sleeptimer);
        
        myApp = (MyApplication)getApplicationContext();

        timePicker1 = (TimePicker)findViewById(R.id.timePicker1);
        textTimer = (TextView)findViewById(R.id.textTimer);
        btnStart = (Button)findViewById(R.id.btnStart);
        btnSleepCancel = (Button)findViewById(R.id.btnSleepCancel);
        btnSleepPauseResume = (Button)findViewById(R.id.btnSleepPauseResume);
        
        btnStart.setOnClickListener(this);
        btnSleepCancel.setOnClickListener(this);
        btnSleepPauseResume.setOnClickListener(this);
        
        timePicker1.setCurrentHour(0);
        timePicker1.setCurrentMinute(15);
        
        setDisplayState();
    }
    
    

    @Override
    protected void onPause() {
        super.onPause();
        
        if(mTimer!=null){
            mTimer.cancel();
            mTimer = null;
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        if(mTimer == null){
            mTimer = new Timer(true);
            mTimer.schedule(new TimerTask() {
                
                @Override
                public void run() {
                    mHandler.post(new Runnable() {
                        @Override
                        public void run() {
                            setDisplayState();
                        }
                    });
                }
            }, 0, 1000);
        }
    }

    private void setDisplayState(){
        if(!myApp.isSetTimer()){
            findViewById(R.id.layoutSleepTimer1).setVisibility(View.VISIBLE);
            findViewById(R.id.layoutSleepTimer2).setVisibility(View.GONE);
        }else{
            findViewById(R.id.layoutSleepTimer1).setVisibility(View.GONE);
            findViewById(R.id.layoutSleepTimer2).setVisibility(View.VISIBLE);
            if(myApp.isTimerOn()){
                btnSleepPauseResume.setText(getString(R.string.lb_sleep_pause));
            }else{
                btnSleepPauseResume.setText(getString(R.string.lb_sleep_resume));
            }
            
            long limit = myApp.getLimit();
            int totalsec = (int)limit/1000;
            int h = totalsec/3600;
            int m = (int)((totalsec-h*60)/60);
            if(m<0) m = 0;
            int s = totalsec - (h*3600 + m*60);
            if(s<0) s = 0;
            
            StringBuilder buf = new StringBuilder();
            buf.append(String.format("%02d", h));
            buf.append(":");
            buf.append(String.format("%02d", m));
            buf.append(":");
            buf.append(String.format("%02d", s));
            textTimer.setText(buf.toString());
        }
    }

    @Override
    public void onClick(View v) {
        if(v.getId() == R.id.btnStart){
            int h = timePicker1.getCurrentHour();
            int m = timePicker1.getCurrentMinute();
            long delay = ((h*3600)+m*60)*1000;
            myApp.startSleepTimer(delay);
        }else if(v.getId() == R.id.btnSleepCancel){
            myApp.cancelSleepTimer();
        }else if(v.getId() == R.id.btnSleepPauseResume){
            if(btnSleepPauseResume.getText().equals(getString(R.string.lb_sleep_pause))){
                myApp.pauseSleepTimer();
            }else{
                myApp.resumeSleepTimer();
            }
        }
    }
}
