package jp.co.kayo.android.localplayer.util.bean;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import jp.co.kayo.android.localplayer.R;
import jp.co.kayo.android.localplayer.consts.MediaConsts;
import jp.co.kayo.android.localplayer.consts.TableConsts;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioAlbum;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioArtist;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioFavorite;
import jp.co.kayo.android.localplayer.consts.MediaConsts.AudioMedia;
import jp.co.kayo.android.localplayer.provider.ContentsUtils;
import jp.co.kayo.android.localplayer.util.ViewCache;
import jp.co.kayo.android.localplayer.util.bean.SearchItem.ITEM_TYPE;

import android.content.ContentUris;
import android.content.Context;
import android.database.Cursor;
import android.support.v4.content.AsyncTaskLoader;

public class FavoriteItemLoader extends AsyncTaskLoader<List<SearchItem>> {
    private int mFavolite;
    private ArrayList<SearchItem> mList;
    private boolean isSDCard;
    private ViewCache mViewCache;

    public FavoriteItemLoader(Context context, ViewCache viewcache, int favolite) {
        super(context);
        mFavolite = favolite;
        this.mViewCache = viewcache;
        isSDCard = ContentsUtils.isSDCard(context);
    }

    @Override
    public List<SearchItem> loadInBackground() {
        return makeList();
    }

    private List<SearchItem> makeList() {
        if (mList == null) {
            mList = new ArrayList<SearchItem>();
        } else {
            mList.clear();
        }

        Map<String, SearchItem> albums = new HashMap<String, SearchItem>();
        Map<String, SearchItem> artists = new HashMap<String, SearchItem>();
        ArrayList<SearchItem> songs = new ArrayList<SearchItem>();
        makeListSongs(songs);
        makeListAlbums(albums);
        makeListArtist(artists);
        if (albums.size() > 0) {
            SearchItem grp = new SearchItem();
            grp.type = ITEM_TYPE.GROUP;
            grp.title = getContext().getString(R.string.lb_tab_albums_name)
                    .concat("(").concat(Integer.toString(albums.size()))
                    .concat(")");
            mList.add(grp);
            mList.addAll(albums.values());
        }
        if (artists.size() > 0) {
            SearchItem grp = new SearchItem();
            grp.type = ITEM_TYPE.GROUP;
            grp.title = getContext().getString(R.string.lb_tab_artist_name)
                    .concat("(").concat(Integer.toString(artists.size()))
                    .concat(")");
            mList.add(grp);
            mList.addAll(artists.values());
        }
        if (songs.size() > 0) {
            SearchItem grp = new SearchItem();
            grp.type = ITEM_TYPE.GROUP;
            grp.title = getContext().getString(R.string.lb_tab_media_name)
                    .concat("(").concat(Integer.toString(songs.size()))
                    .concat(")");
            mList.add(grp);
            mList.addAll(songs);
        }

        return mList;
    }

    void makeListSongs(ArrayList<SearchItem> songs) {
        if (isSDCard) {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.FAVORITE_CONTENT_URI,
                        new String[] {
                            AudioFavorite.MEDIA_ID
                        },
                        AudioFavorite.POINT + " >= ? and " + AudioFavorite.TYPE
                                + " = ?",
                        new String[] {
                                Integer.toString(mFavolite), TableConsts.FAVORITE_TYPE_SONG
                        }, null);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        long mediaId = cur.getLong(0);

                        Cursor mediacur = null;
                        try {

                            mediacur = getContext().getContentResolver().query(
                                    ContentUris.withAppendedId(MediaConsts.MEDIA_CONTENT_URI,
                                            mediaId),
                                    null, null, null, null);

                            if (mediacur != null && mediacur.moveToFirst()) {
                                String albumKey = mediacur.getString(mediacur
                                        .getColumnIndex(AudioMedia.ALBUM_KEY));
                                String artistKey = mediacur.getString(mediacur
                                        .getColumnIndex(AudioMedia.ARTIST_KEY));
                                String album = mediacur.getString(mediacur
                                        .getColumnIndex(AudioMedia.ALBUM));
                                String artist = mediacur.getString(mediacur
                                        .getColumnIndex(AudioMedia.ARTIST));
                                String title = mediacur.getString(mediacur
                                        .getColumnIndex(AudioMedia.TITLE));
                                FavoriteInfo inf = mViewCache.getFavorite(getContext(), mediaId, TableConsts.FAVORITE_TYPE_SONG);

                                SearchItem item = new SearchItem();
                                item.id = mediacur.getLong(mediacur.getColumnIndex(AudioMedia._ID));
                                item.type = ITEM_TYPE.SONG;
                                item.albumKey = albumKey;
                                item.artistKey = artistKey;
                                item.album = album;
                                item.artist = artist;
                                item.title = title;
                                item.duration = mediacur.getLong(mediacur
                                        .getColumnIndex(AudioMedia.DURATION));
                                item.data = mediacur.getString(mediacur
                                        .getColumnIndex(AudioMedia.DATA));
                                if(inf != null){
                                    item.rating = inf.rating;
                                }
                                songs.add(item);
                            }
                        } finally {
                            if (mediacur != null) {
                                mediacur.close();
                            }
                        }

                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }

        } else {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.MEDIA_CONTENT_URI, null,
                        TableConsts.FAVORITE_POINT + " >= ?",
                        new String[] {
                            Integer.toString(mFavolite)
                        },
                        MediaConsts.AudioMedia.TITLE);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        String albumKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM_KEY));
                        String artistKey = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST_KEY));
                        String album = cur.getString(cur
                                .getColumnIndex(AudioMedia.ALBUM));
                        String artist = cur.getString(cur
                                .getColumnIndex(AudioMedia.ARTIST));
                        String title = cur.getString(cur
                                .getColumnIndex(AudioMedia.TITLE));
                        int rating = cur.getInt(cur.getColumnIndex(MediaConsts.AudioMedia.FAVORITE_POINT));

                        SearchItem item = new SearchItem();
                        item.id = cur.getLong(cur.getColumnIndex(AudioMedia._ID));
                        item.type = ITEM_TYPE.SONG;
                        item.albumKey = albumKey;
                        item.artistKey = artistKey;
                        item.album = album;
                        item.artist = artist;
                        item.title = title;
                        item.duration = cur.getLong(cur
                                .getColumnIndex(AudioMedia.DURATION));
                        item.data = cur.getString(cur
                                .getColumnIndex(AudioMedia.DATA));
                        item.rating = rating;
                        songs.add(item);
                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    void makeListAlbums(Map<String, SearchItem> albums) {
        if (isSDCard) {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.FAVORITE_CONTENT_URI,
                        new String[] {
                            AudioFavorite.MEDIA_ID
                        },
                        AudioFavorite.POINT + " >= ? and " + AudioFavorite.TYPE
                                + " = ?",
                        new String[] {
                                Integer.toString(mFavolite), TableConsts.FAVORITE_TYPE_ALBUM
                        }, null);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        long mediaId = cur.getLong(0);

                        Cursor mediacur = null;
                        try {

                            mediacur = getContext().getContentResolver().query(
                                    ContentUris.withAppendedId(MediaConsts.ALBUM_CONTENT_URI,
                                            mediaId),
                                    new String[] {
                                            AudioAlbum.ALBUM_KEY, AudioAlbum.ALBUM,
                                            AudioAlbum.ARTIST
                                    }, null, null, null);

                            if (mediacur != null && mediacur.moveToFirst()) {
                                String albumKey = mediacur.getString(mediacur
                                        .getColumnIndex(AudioAlbum.ALBUM_KEY));
                                String album = mediacur.getString(mediacur
                                        .getColumnIndex(AudioAlbum.ALBUM));
                                String artist = mediacur.getString(mediacur
                                        .getColumnIndex(AudioAlbum.ARTIST));
                                FavoriteInfo inf = mViewCache.getFavorite(getContext(), mediaId, TableConsts.FAVORITE_TYPE_ALBUM);

                                SearchItem item = new SearchItem();
                                item.type = ITEM_TYPE.ALBUM;
                                item.albumKey = albumKey;
                                item.album = album;
                                item.artist = artist;
                                if(inf != null){
                                    item.rating = inf.rating;
                                }
                                albums.put(item.albumKey, item);
                            }
                        } finally {
                            if (mediacur != null) {
                                mediacur.close();
                            }
                        }

                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }

        }
        else {
            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ALBUM_CONTENT_URI,
                        new String[] {
                                AudioAlbum.ALBUM_KEY, AudioAlbum.ALBUM,
                                AudioAlbum.ARTIST
                        }, TableConsts.FAVORITE_POINT + " >= ?",
                        new String[] {
                            Integer.toString(mFavolite)
                        },
                        MediaConsts.AudioAlbum.ALBUM);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        String albumKey = cur.getString(cur
                                .getColumnIndex(AudioAlbum.ALBUM_KEY));
                        String album = cur.getString(cur
                                .getColumnIndex(AudioAlbum.ALBUM));
                        String artist = cur.getString(cur
                                .getColumnIndex(AudioAlbum.ARTIST));
                        int rating = cur.getInt(cur.getColumnIndex(MediaConsts.AudioAlbum.FAVORITE_POINT));

                        SearchItem item = new SearchItem();
                        item.type = ITEM_TYPE.ALBUM;
                        item.albumKey = albumKey;
                        item.album = album;
                        item.artist = artist;
                        item.rating = rating;
                        albums.put(item.albumKey, item);
                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    void makeListArtist(Map<String, SearchItem> artists) {
        if (isSDCard) {
            Cursor cur = null;
            try {

                cur = getContext().getContentResolver().query(
                        MediaConsts.FAVORITE_CONTENT_URI,
                        new String[] {
                            AudioFavorite.MEDIA_ID
                        },
                        AudioFavorite.POINT + " >= ? and " + AudioFavorite.TYPE
                                + " = ?",
                        new String[] {
                                Integer.toString(mFavolite), TableConsts.FAVORITE_TYPE_ARTIST
                        }, null);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        long mediaId = cur.getLong(0);

                        Cursor mediacur = null;
                        try {

                            mediacur = getContext().getContentResolver().query(
                                    ContentUris.withAppendedId(MediaConsts.ARTIST_CONTENT_URI,
                                            mediaId),
                                    new String[] {
                                            AudioArtist.ARTIST_KEY, AudioArtist.ARTIST,
                                            AudioArtist.NUMBER_OF_TRACKS
                                    }, null, null, null);

                            if (mediacur != null && mediacur.moveToFirst()) {
                                String artistKey = mediacur.getString(mediacur
                                        .getColumnIndex(AudioArtist.ARTIST_KEY));
                                String artist = mediacur.getString(mediacur
                                        .getColumnIndex(AudioArtist.ARTIST));
                                int max = mediacur.getInt(mediacur
                                        .getColumnIndex(AudioArtist.NUMBER_OF_TRACKS));
                                FavoriteInfo inf = mViewCache.getFavorite(getContext(), mediaId, TableConsts.FAVORITE_TYPE_ARTIST);

                                SearchItem item = new SearchItem();
                                item.type = ITEM_TYPE.ARTIST;
                                item.artistKey = artistKey;
                                item.artist = artist;
                                item.number_of_tracks = max;
                                if(inf != null){
                                    item.rating = inf.rating;
                                }
                                artists.put(item.artistKey, item);
                            }
                        } finally {
                            if (mediacur != null) {
                                mediacur.close();
                            }
                        }

                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }

        }
        else {

            Cursor cur = null;
            try {
                cur = getContext().getContentResolver().query(
                        MediaConsts.ARTIST_CONTENT_URI,
                        new String[] {
                                AudioArtist.ARTIST_KEY, AudioArtist.ARTIST,
                                AudioArtist.NUMBER_OF_TRACKS
                        }, TableConsts.FAVORITE_POINT + " >= ?",
                        new String[] {
                            Integer.toString(mFavolite)
                        },
                        MediaConsts.AudioArtist.ARTIST);

                if (cur != null && cur.moveToFirst()) {
                    do {
                        String artistKey = cur.getString(cur
                                .getColumnIndex(AudioArtist.ARTIST_KEY));
                        String artist = cur.getString(cur
                                .getColumnIndex(AudioArtist.ARTIST));
                        int max = cur.getInt(cur
                                .getColumnIndex(AudioArtist.NUMBER_OF_TRACKS));
                        int rating = cur.getInt(cur.getColumnIndex(MediaConsts.AudioArtist.FAVORITE_POINT));

                        SearchItem item = new SearchItem();
                        item.type = ITEM_TYPE.ARTIST;
                        item.artistKey = artistKey;
                        item.artist = artist;
                        item.number_of_tracks = max;
                        item.rating = rating;
                        artists.put(item.artistKey, item);

                    } while (cur.moveToNext());
                }
            } finally {
                if (cur != null) {
                    cur.close();
                }
            }
        }
    }

    @Override
    protected void onStartLoading() {
        super.onStartLoading();
        if (mList == null) {
            forceLoad();
        } else {
            deliverResult(mList);
        }
    }

}
